from typing import Annotated

import typer

from ..utils.ado import execute_pipeline

app = typer.Typer()


@app.command(
    name="teardown_inactive_cluster",
    help="清理非活动集群",
    short_help="清理非活动集群"
)
def teardown_inactive_cluster(
        release: Annotated[str, typer.Argument(
            help="发布版本号",
            show_default=False,
        )]
) -> None:
    """
    清理非活动集群的资源。

    此命令用于触发 Azure DevOps 流水线来清理非活动集群的所有资源。
    清理操作将移除指定版本的所有相关资源。

    Args:
        release: 要清理的版本号，例如 "1.0.0"

    环境变量:
        DEPLOY_PIPELINES_TEARDOWN_INACTIVE_CLUSTER_NAME: 清理流水线名称
        DEPLOY_PIPELINES_TEARDOWN_INACTIVE_CLUSTER_PARAMETERS: 清理流水线参数配置

    Examples:
        清理指定版本的非活动集群：
            $ hacli deploy teardown_inactive_cluster 1.0.0

    注意:
        - 此操作将清理所有相关资源，请谨慎执行
        - 确保指定的版本号正确
        - 执行前请确保已正确设置所需的环境变量
    """
    execute_pipeline(release=release,
                     pipeline_name_env_key="DEPLOY_PIPELINES_TEARDOWN_INACTIVE_CLUSTER_NAME",
                     pipeline_parameter_env_key="DEPLOY_PIPELINES_TEARDOWN_INACTIVE_CLUSTER_PARAMETERS")



from fastapi import APIRouter

from hacli.tag import get_global_rollout_status as rollout_status

router = APIRouter()


@router.get("/rollout/tags",
            tags=["Tag"],
            description="""
             ## 获取 global 当前 appr 各个服务的 release tag
""")
async def get_tags(
) -> dict[str, str]:
    return rollout_status.get_global_rollout_status()



from typing import Union

import typer
from typing_extensions import Annotated

from ..utils.ado import execute_pipeline

app = typer.Typer()


@app.command(
    name="deploy_inactive_shared_services",
    help="部署共享服务到非活动集群",
    short_help="部署共享服务"
)
def deploy_inactive_shared_services(
        release: Annotated[str, typer.Argument(
            help="发布版本号",
            show_default=False,
            metavar="VERSION"
        )],
        tag: Annotated[str, typer.Argument(
            help="部署标签",
            show_default=False,
            metavar="TAG"
        )]
) -> Union[dict, None]:
    """
    将共享服务部署到非活动集群。

    此命令用于将共享服务部署到非活动集群中。部署时需要指定版本号和标签。

    Args:
        release: 要部署的版本号，例如 "1.0.0"
        tag: 部署使用的标签，用于标识部署目标

    环境变量:
        DEPLOY_PIPELINES_DEPLOY_INACTIVE_SHARED_SERVICES_NAME: 部署流水线名称
        DEPLOY_PIPELINES_DEPLOY_INACTIVE_SHARED_SERVICES_PARAMETERS: 部署流水线参数配置

    Examples:
        部署指定版本的共享服务：
            $ hacli deploy deploy_inactive_shared_services 1.0.0 prod-tag

    注意:
        - 确保提供的版本号在代码仓库中存在
        - 确保标签与目标环境匹配
        - 执行前请确保已正确设置所需的环境变量
    """
    return execute_pipeline(release=release,
                     pipeline_name_env_key="DEPLOY_PIPELINES_DEPLOY_INACTIVE_SHARED_SERVICES_NAME",
                     pipeline_parameter_env_key="DEPLOY_PIPELINES_DEPLOY_INACTIVE_SHARED_SERVICES_PARAMETERS",
                     tag=tag)



import os
from enum import Enum
from pathlib import Path
from typing import Union

from pydantic import AnyHttpUrl
from pydantic_settings import BaseSettings

IS_DEV = True  # 是否开发环境

current_path = Path().absolute().parent  # 当前路径


# https://docs.pydantic.dev/usage/settings/
class Settings(BaseSettings):
    PROJECT_DESC: str = "🎉 接口汇总 🎉"  # 描述
    PROJECT_VERSION: Union[int, str] = 1.0  # 版本
    API_PREFIX: str = "/api"  # 接口前缀

    STATIC_DIR: str = "static"  # 静态文件目录
    BASE_URL: AnyHttpUrl = "http://127.0.0.1:8000"  # 开发环境(为了存放图片全路径)
    CORS_ORIGINS: list[AnyHttpUrl] = ["http://127.0.0.1:5173"]  # 跨域请求(务必指定精确ip, 不要用localhost)

    MD5_SALT: str = "9iJvchvS"  # md5 加密盐
    COOKIE_KEY: str = "sessionId"  # Cookie key name
    COOKIE_MAX_AGE: int = 24 * 60 * 60  # Cookie 有效时间
    COOKIE_NOT_CHECK: list[str] = ["/api/user/login", "/api/user/signup"]  # 不校验 Cookie

    REDIS_URI: str = "redis://:123456@localhost:6379/0"  # Redis
    REDIS_EXPIRE: int = 24 * 60 * 60  # Redis 过期时长
    REDIS_GLOBAL_PREFIX: str = "redis-om"  # Redis 全局前缀
    DATABASE_URI: str = "mysql://root:123456@localhost:3306/demo?charset=utf8"  # MySQL
    DATABASE_ECHO: bool = False  # 是否打印数据库日志 (可看到创建表、表数据增删改查的信息)

    LOGGER_DIR: str = "logs"  # 日志文件夹名
    LOGGER_NAME: str = '{time:YYYY-MM-DD_HH-mm-ss}.log'  # 日志文件名 (时间格式)
    LOGGER_LEVEL: str = 'DEBUG'  # 日志等级: ['DEBUG' | 'INFO']
    LOGGER_ROTATION: str = "00:00"  # 日志分片: 按 时间段/文件大小 切分日志. 例如 ["500 MB" | "12:00" | "1 week"]
    LOGGER_RETENTION: str = "30 days"  # 日志保留的时间: 超出将删除最早的日志. 例如 ["1 days"]

    class Config:
        case_sensitive = True  # 区分大小写


class DevelopmentConfig(Settings):
    pass


class ProductionConfig(Settings):
    BASE_URL: AnyHttpUrl = "http://114.115.143.81:8000"  # 生产环境(为了存放图片全路径)
    CORS_ORIGINS: list[AnyHttpUrl] = ["http://114.115.143.81"]  # 跨域请求

    REDIS_URI: str = "redis://:123456@redis:6379/0"  # Redis
    DATABASE_URI: str = "mysql://root:123456@mysql:3306/demo?charset=utf8"  # MySQL
    DATABASE_ECHO: bool = True  # 是否打印数据库日志 (可看到创建表、表数据增删改查的信息)

    LOGGER_LEVEL: str = 'INFO'  # 日志等级: ['DEBUG' | 'INFO']


settings = DevelopmentConfig() if IS_DEV else ProductionConfig()


class FileDirEnum(Enum):
    """ 文件类型枚举 """
    AVATAR = os.path.join(settings.STATIC_DIR, "avatar")  # 头像文件目录
    ICON = os.path.join(settings.STATIC_DIR, "icon")  # Icon文件目录



from typing import Union

import typer

from ..utils.ado import execute_pipeline

app = typer.Typer()


@app.command(
    name="purge_local_images",
    help="清理本地容器注册表中的镜像",
    short_help="清理本地镜像"
)
def purge_local_images() -> Union[dict, None]:
    """
    清理本地容器注册表中的旧镜像。

    此命令会触发 Azure DevOps 流水线来清理本地容器注册表中的旧镜像。
    清理操作基于预设的策略进行，通常会保留最新的几个版本。

    环境变量:
        DEPLOY_PIPELINES_PURGE_LOCAL_IMAGES_NAME: 清理流水线名称
        DEPLOY_PIPELINES_PURGE_LOCAL_IMAGES_PARAMETERS: 清理流水线参数配置

    Examples:
        清理本地镜像：
            $ hacli image purge_local_images

    注意:
        - 此操作不可逆，请确保要清理的镜像确实不再需要
        - 确保已正确设置所需的环境变量
        - 清理过程可能需要一些时间，具体取决于镜像数量
    """

    return execute_pipeline(
        release="master",
        pipeline_name_env_key="DEPLOY_PIPELINES_PURGE_LOCAL_IMAGES_NAME",
        pipeline_parameter_env_key="DEPLOY_PIPELINES_PURGE_LOCAL_IMAGES_PARAMETERS"
    )



import os

import typer
from rich import print
from typing_extensions import Annotated

from ..utils.git_commands import LocalGitCommand

app = typer.Typer()


@app.command(
    name="create_local_repo_branch",
    help="创建一个新的 Git 分支",
    short_help="创建新分支"
)
def create_local_repo_branch(
        branch: Annotated[str, typer.Argument(
            help="新分支的名称",
            show_default=False,
        )],
        base_branch: Annotated[str, typer.Option(
            help="基础分支名称",
            show_default=True,
        )] = "master",
        is_global: Annotated[bool, typer.Option(
            "--global",
            "-g",
            help="是否在全局 Git 仓库中创建分支",
            show_default=True
        )] = False,
):
    """
    创建一个新的 Git 分支。
    Args:
        branch: 新分支的名称
        base_branch: 基础分支名称，默认为 master
        is_global: 是否为全局分支
    """
    git_cmd = LocalGitCommand(is_global=is_global)
    git_cmd.branch.create_branch(branch, base_branch)
    print(
        f"Branch [bold green]{branch}[/bold green] created from [bold green]{base_branch}[/bold green] in dir [bold green]{git_cmd.working_dir}[/bold green]")


@app.command(
    name="create_local_release_branch",
    help="创建一个新的 Release 分支",
    short_help="创建本地Release分支"
)
def create_local_release_branch(
        release: Annotated[str, typer.Argument(
            help="release",
            show_default=False,
        )]
):
    """
    创建一个新的 Git 分支。
    Args:
        release: 新分支的名称
    """
    git_cmd = LocalGitCommand(is_global=False)
    release_branch = os.environ["RELEASE_GIT_LOCAL_RELEASE_NAME_TEMPLATE"].format(release=release)
    git_cmd.branch.create_branch(release_branch, "master")
    print(
        f"Branch [bold green]{release_branch}[/bold green] created from [bold green]master[/bold green] in dir [bold green]{git_cmd.working_dir}[/bold green]")


@app.command(
    name="create_global_rollout_update_branch",
    help="创建一个新的 Application Rollout Update 分支",
    short_help="创建Global Rollout Update分支"
)
def create_global_rollout_update_branch(
        release: Annotated[str, typer.Argument(
            help="release",
            show_default=False,
        )],
        task_id: Annotated[str, typer.Argument(
            help="task_id",
            show_default=False,
        )]
):
    """
    创建一个新的 Git 分支。
    Args:
        release: 新分支的名称
        task_id: work item id
    """
    # TODO remove work_item id 自动创建
    # Task 355970: Application Rollout Update in CN 25.1.3 的 name
    git_cmd = LocalGitCommand(is_global=True)
    release_branch = f"feature/{task_id}-Application-Rollout-Update-in-CN-{release}"
    git_cmd.branch.create_branch(release_branch, "master")
    print(
        f"Branch [bold green]{release_branch}[/bold green] created from [bold green]master[/bold green] in dir [bold green]{git_cmd.working_dir}[/bold green]")



import json
import os

import typer
from rich import print
from tabulate import tabulate
from typing_extensions import Annotated

from hacli.utils.git_commands import LocalGitCommand

app = typer.Typer()


@app.command(name="release_checkout_date",
             help="release 从 master checkout 的 时间， 和分支的相关信息",
             short_help="release 创建的日期，分支信息")
def release_checkout_date(
        release: Annotated[str, typer.Argument(help="版本号，例如25.1.1")],
):
    git_cmd = LocalGitCommand(True)

    service_name_convert_mappings = json.loads(os.environ["TAG_SERVICE_NAME_CONVERT_MAPPINGS"])

    table_data = []
    headers = ["Service", "Version", "Checkout Date", "Message", "Hot Fix"]

    for product in service_name_convert_mappings.values():
        branch_name = os.environ["RELEASE_NOTE_BRANCH_NAME_TEMPLATE"].format(product=product, release=release)

        if not git_cmd.branch.branch_exists(branch_name):
            table_data.append([product, f"Branch {branch_name} Not Found", "-", "-", "-"])
            continue

        merge_base = git_cmd.branch.get_merge_base(branch_name, "master")
        if not merge_base:
            table_data.append([product, "Merge base not found", "-", "-", "-"])
            continue

        commit_date, commit_message = git_cmd.commit.get_latest_commit_info(merge_base)
        if not commit_date or not commit_message:
            table_data.append([product, "No commits found", "-", "-", "-"])
            continue

        # 判断是否有 Hot Fix
        new_commits = git_cmd.commit.has_hotfixes(merge_base, branch_name)

        hotfix_info = f"Yes ({len(new_commits)})" if new_commits else "No"

        # 添加到表格数据
        table_data.append([product, branch_name, f"`{commit_date}`", commit_message.strip(), hotfix_info])

    # 生成 Markdown 格式表格
    markdown_table = tabulate(table_data, headers=headers, tablefmt="github")

    # 保存到 Markdown 文件
    with open("Product Release Checkout Date.md", "a+", encoding="utf-8") as f:
        f.write(f"\n# Product Release {release} Checkout Date\n")
        f.write(markdown_table + "\n")

    # 打印 Markdown 表格
    res = '\n'.join([f"# Product Release {release} Checkout Report\n", markdown_table])
    print(res)
    return res



import typer
from typing_extensions import Union

from ..utils.ado import execute_pipeline

app = typer.Typer()


@app.command(
    name="import_global_external_images",
    help="导入全局外部镜像到容器注册表",
    short_help="导入外部镜像"
)
def import_global_external_images() -> Union[dict, None]:
    """
    从外部源导入全局镜像到 Azure 容器注册表。

    此命令会触发 Azure DevOps 流水线来导入全局外部镜像。
    导入过程基于全局仓库的 master 分支配置进行。

    环境变量:
        DEPLOY_PIPELINES_IMPORT_GLOBAL_EXTERNAL_IMAGES_NAME: 导入流水线名称
        DEPLOY_PIPELINES_IMPORT_GLOBAL_EXTERNAL_IMAGES_PARAMETERS: 导入流水线参数
        PROJECT_GLOBAL_NAME: 全局项目名称
        PROJECT_GLOBAL_REPO: 全局仓库名称

    Examples:
        导入全局外部镜像：
            $ hacli image import_global_external_images

    注意:
        - 确保已正确设置所有必需的环境变量
        - 确保有权限访问全局仓库和目标容器注册表
        - 此操作可能需要较长时间，具体取决于镜像大小和数量
    """
    return execute_pipeline(release="master",
                            pipeline_name_env_key="DEPLOY_PIPELINES_IMPORT_GLOBAL_EXTERNAL_IMAGES_NAME",
                            pipeline_parameter_env_key="DEPLOY_PIPELINES_IMPORT_GLOBAL_EXTERNAL_IMAGES_PARAMETERS",
                            project_name_env_key="PROJECT_GLOBAL_NAME",
                            project_repo_env_key="PROJECT_GLOBAL_REPO")



# from fastapi import FastAPI, Request
# from fastapi import Query
# from fastapi.responses import JSONResponse
# from fastapi.responses import PlainTextResponse
# from typing_extensions import Dict, Union, Any
#
# from hacli.apis.schemas.git import CreateBranchRequest
# from hacli.release.create_release_note_md import create_release_note_md
#
# app = FastAPI(
#     title="Hacli API",
#     description="Hacli REST API Service",
#     version="1.0.0"
# )
#
#
#
#
# # from hacli.deploy import check_services_deployment_status as deployment_status
# #
# #
# # @app.get("/check/deployment/status", tags=["Deploy"], description="""
# #             ## 检查当前部署是否已经完成, 请确保 `az login`已登录
# # """)
# # async def check_services_deployment_status(
# # ) -> Dict[str, str]:
# #     return deployment_status.check_services_deployment_status()
#
#
#
#
# from hacli.git import create_local_repo_branch as git_ops
#
#
# @app.post(
#     "/check/deployment/status", )
# def create_repo_branch(request: CreateBranchRequest):
#     return git_ops.create_local_repo_branch(request.branch, request.base_branch, request.is_global)
#
#
# # TODO use cache if possible
# @app.get("/release/notes", tags=["Release"], response_class=PlainTextResponse)
# async def get_release_notes(
#         pre_release: str = Query(
#             ...,
#             title="Previous Release",
#             description="前一个版本的标签名称",
#             example="1.0.0",
#             min_length=1
#         ),
#         cur_release: str = Query(
#             ...,
#             title="Current Release",
#             description="当前版本的标签名称",
#             example="1.1.0",
#             min_length=1
#         )
# ) -> str:
#     """
#         生成版本发布说明文档（Release Notes）的 md 文件
#     """
#     return create_release_note_md(pre_release=pre_release, cur_release=cur_release)
#
#
#
#
# # from hacli.image import (
# #     check_service_image_imported as check_image,
# #     purge_local_images as purge,
# #     import_global_service_images as service_images,
# #     import_global_external_images as external_images,
# # )
# #
# # @app.get("/check", tags=["Image"], description="""
# #             ## 检查各个产品的服务是否已经导入, 请确保 `az login`已登录
# # """)
# # async def check_service_image_imported(
# # ) -> Dict[str, Union[str, Dict[str, str]]]:
# #     return check_image.check_service_image_imported()
# #
# #
# # @app.get("/purge/local", tags=["Image"], description="""
# #             ## Purge 本地 acr 镜像, 请确保 `az login`已登录
# #             请在 terminal 确认是否要执行 pipeline
# # """)
# # async def purge_local_images(
# # ):
# #     return purge.purge_local_images()
# #
# #
# # @app.get("/import/global/service", tags=["Image"], description="""
# #             ## 导入 global 各个 product 对应 release tag 的镜像
# #             请在 terminal 确认是否要执行 pipeline
# # """)
# # async def import_global_service(
# # ) -> dict[str, Any]:
# #     return service_images.import_global_service_images()
# #
# #
# # @app.get("/import/external", tags=["Image"], description="""
# #             ## 导入 global 基础的第三方镜像
# #             请在 terminal 确认是否要执行 pipeline
# # """)
# # async def import_global_external(
# # ) -> Union[dict, None]:
# #     return external_images.import_global_external_images()
#
#
# # from hacli.tag.get_global_rollout_status import get_global_rollout_status
# #
# #
# # @app.get("/rollout/tags",
# #          tags=["Tag"],
# #          description="""
# #              ## 获取 global 当前 appr 各个服务的 release tag
# # """)
# # async def get_tags(
# # ) -> dict[str, str]:
# #     return get_global_rollout_status()



from pydantic import BaseModel, Field


class CreateBranchRequest(BaseModel):
    branch: str = Field(..., description="新分支的名称", example="feature/new-branch")
    base_branch: str = Field("master", description="基础分支名称", example="develop")
    is_global: bool = Field(False, description="是否在全局 Git 仓库中创建分支", example=True)



import json
import os
from datetime import datetime
from string import Template

import typer
from git import GitCommandError
from rich import print
from typing_extensions import Annotated

from hacli.release.create_release_note_md import add_configuration_pr_link
from hacli.utils.git_commands import LocalGitCommand

app = typer.Typer()


@app.command(name="configuration_change_list",
             help=" 生成版本间配置文件变化的 md 文件",
             short_help="生成版本间配置变化文档")
def configuration_change_list(
        pre_release: Annotated[str, typer.Argument(help="前一个版本号，例如：1.0.0")],
        cur_release: Annotated[str, typer.Argument(help="当前版本号，例如：1.1.0")]
):
    configuration_changes = []
    service_name_convert_mappings = json.loads(os.environ["TAG_SERVICE_NAME_CONVERT_MAPPINGS"])
    configuration_changes.append("## Configuration Change List\n")
    command = LocalGitCommand(True).repo
    for product in service_name_convert_mappings.values():
        commands = (
            Template(os.environ["RELEASE_NOTE_CONFIGURATION_GIT_COMMAND"])
            .safe_substitute(product=product,
                             pre_release=pre_release,
                             cur_release=cur_release)
        )
        formatted_product = product.replace("-", " ").title()
        print("Start to generate [red]configuration changes[/red] for product: [red]{}[/red]".format(formatted_product))
        try:
            configuration_changes.append(f"\n### {formatted_product}\n")
            print("    >>> Command for [red]{formatted_product}[/red]: ", commands)
            log_output = command.git.execute(command=commands, shell=True)
            configuration_changes.append(add_configuration_pr_link(log_output))
        except GitCommandError as e:
            configuration_changes.append(e.stderr)

    final_str = '\n'.join(configuration_changes)
    print("---------------------------- [green] End Generation [/green] ----------------------------")
    print("Final MD Content: \n", final_str)
    now = datetime.now().strftime("%Y-%m-%d-%H:%M:%S")
    file_name = f"Configuration Change List [{pre_release}-{cur_release}]-{now}"
    with open(f"{file_name}.md", "w", encoding="utf-8") as f:
        f.write(final_str)

    return final_str



from typing import Dict, Union, Any

from fastapi import APIRouter

router = APIRouter()

from hacli.image import (
    check_service_image_imported as check_image,
    purge_local_images as purge,
    import_global_service_images as service_images,
    import_global_external_images as external_images,
)


@router.get("/check", tags=["Image"], description="""
            ## 检查各个产品的服务是否已经导入, 请确保 `az login`已登录
""")
async def check_service_image_imported(
) -> Dict[str, Union[str, Dict[str, str]]]:
    return check_image.check_service_image_imported()


@router.get("/purge/local", tags=["Image"], description="""
           ## 请在 terminal 确认是否要执行 pipeline
""")
async def purge_local_images(
):
    return purge.purge_local_images()


@router.get("/import/global/service", tags=["Image"], description="""
            ## 导入 global 各个 product 对应 release tag 的镜像
            请在 terminal 确认是否要执行 pipeline
""")
async def import_global_service(
) -> dict[str, Any]:
    return service_images.import_global_service_images()


@router.get("/import/external", tags=["Image"], description="""
            ## 导入 global 基础的第三方镜像
            请在 terminal 确认是否要执行 pipeline
""")
async def import_global_external(
) -> Union[dict, None]:
    return external_images.import_global_external_images()



import json
import os
from os import environ
from string import Template
from typing import Union, List

import typer
from azure.devops.connection import Connection
from azure.devops.exceptions import AzureDevOpsServiceError
from azure.devops.v7_0.dashboard import TeamContext, Dashboard, DashboardClient, Widget
from azure.devops.v7_0.git import GitClient
from azure.devops.v7_0.pipelines import PipelinesClient, Pipeline, Run
from azure.devops.v7_0.work_item_tracking import WorkItemTrackingClient, WorkItemReference, Wiql, WorkItemQueryResult, \
    WorkItem
from msrest.authentication import BasicAuthentication
from rich.prompt import Confirm


class ADOPipelineClient:
    """Azure DevOps 流水线客户端"""

    def __init__(self, client: PipelinesClient, project: str):
        self.client = client
        self.project = project

    def trigger_pipeline(self, pipeline_id: int, run_parameters: dict) -> Run:
        """触发流水线执行"""
        return self.client.run_pipeline(
            pipeline_id=pipeline_id,
            project=self.project,
            run_parameters=run_parameters
        )

    def get_pipeline(self, pipeline_id: int) -> Pipeline:
        """获取指定 ID 的流水线"""
        return self.client.get_pipeline(self.project, pipeline_id=pipeline_id)

    def list_pipelines(self) -> list[Pipeline]:
        """列出所有流水线"""
        return self.client.list_pipelines(self.project, top=9999)

    def get_pipeline_by_name(self, pipeline_name: str) -> Union[Pipeline, None]:
        """根据名称获取流水线"""
        pipelines: list[Pipeline] = self.list_pipelines()
        for pipeline in pipelines:
            if pipeline.name == pipeline_name:
                return pipeline
        return None


class ADODashboardClient:
    """Azure DevOps 仪表板客户端"""

    def __init__(self, client: DashboardClient, project: str):
        self.client = client
        self.project = project

    def get_dashboard(self, team: str, dashboard_id: str) -> Dashboard:
        """获取指定团队的仪表板"""
        team_context = TeamContext(project=self.project, team=team)
        return self.client.get_dashboard(team_context, dashboard_id=dashboard_id)

    def get_widget_from_dashboard(self, team: str, dashboard_id: str, widget_id: str) -> Widget:
        """获取仪表板中的小部件"""
        team_context = TeamContext(project=self.project, team=team)
        return self.client.get_widget(team_context, dashboard_id=dashboard_id, widget_id=widget_id)


class ADOGitClient:
    """Azure DevOps Git 客户端"""

    def __init__(self, client: GitClient, project: str):
        self.client = client
        self.project = project

    def check_release_branch(self, repository_id: str, release_branch_name: str) -> bool:
        """检查发布分支是否存在"""
        try:
            self.client.get_branch(
                repository_id=repository_id,
                name=release_branch_name,
                project=self.project
            )
            return True
        except AzureDevOpsServiceError:
            return False


class ADOWorkItemTrackingClient:

    def __init__(self, client: WorkItemTrackingClient, project: str):
        self.client = client
        self.project = project
        self.team_context = TeamContext(self.project)

    def query_first_work_item_reference(self, wiql_query: str) -> Union[WorkItemReference, None]:
        try:
            wiql = Wiql(query=wiql_query)
            query_result: WorkItemQueryResult = self.client.query_by_wiql(wiql, team_context=self.team_context, top=1)
            if query_result.work_items:
                return query_result.work_items[0]
            return None
        except AzureDevOpsServiceError:
            return None

    def query_work_item_reference(self, wiql_query: str) -> Union[List[WorkItemReference], None]:
        try:
            wiql = Wiql(query=wiql_query)
            query_result: WorkItemQueryResult = self.client.query_by_wiql(wiql, team_context=self.team_context)
            if query_result.work_items:
                return query_result.work_items
            return None
        except AzureDevOpsServiceError:
            return None

    def query_working_items(self, work_item_ids: list[int], fields: list[str]) -> list[WorkItem]:
        try:
            return self.client.get_work_items(work_item_ids, project=self.project, fields=fields)
        except AzureDevOpsServiceError:
            return []

    def query_working_item(self, work_item_id: int, fields: list[str]) -> Union[WorkItem, None]:
        try:
            return self.client.get_work_item(work_item_id, project=self.project, fields=fields)
        except AzureDevOpsServiceError:
            return None


class AzureDevOpsClient:
    """Azure DevOps 客户端主类"""

    def __init__(self, project: str):
        self.org_url = environ["ORG_URL"]
        self.personal_access_token = environ["PERSONAL_ACCESS_TOKEN"]
        self.project = project

        # 初始化连接
        self.credentials = BasicAuthentication('', self.personal_access_token)
        self.connection = Connection(base_url=self.org_url, creds=self.credentials)

        # 初始化子客户端
        self.pipeline = ADOPipelineClient(
            self.connection.clients.get_pipelines_client(),
            self.project
        )
        self.dashboard = ADODashboardClient(
            self.connection.clients.get_dashboard_client(),
            self.project
        )
        self.git = ADOGitClient(
            self.connection.clients.get_git_client(),
            self.project
        )
        self.work_items = ADOWorkItemTrackingClient(
            self.connection.clients.get_work_item_tracking_client(),
            self.project
        )


def execute_pipeline(
        release: str,
        pipeline_name_env_key: str,
        pipeline_parameter_env_key: str,
        project_name_env_key: str = "PROJECT_LOCAL_NAME",
        project_repo_env_key: str = "PROJECT_LOCAL_REPO",
        **kwargs
) -> Union[dict, None]:
    """
    执行 Azure DevOps 流水线。

    Args:
        release: 发布版本
        pipeline_name_env_key: 流水线名称环境变量键
        pipeline_parameter_env_key: 流水线参数环境变量键
        project_name_env_key: 项目名称环境变量键
        project_repo_env_key: 项目仓库环境变量键
        **kwargs: 其他参数
    """
    kwargs["release"] = release
    client = AzureDevOpsClient(environ[project_name_env_key])

    # 检查发布分支是否存在
    release_branch_name = release if release == "master" else os.environ[
        "RELEASE_GIT_LOCAL_RELEASE_NAME_TEMPLATE"].format(release=release)
    release_branch_exist = client.git.check_release_branch(
        environ[project_repo_env_key],
        release_branch_name
    )
    if not release_branch_exist:
        raise Exception(f"Release branch [red]{release_branch_name}[/red] does not exist")

    # 获取并检查流水线
    pipeline_name = Template(environ[pipeline_name_env_key]).safe_substitute(**kwargs)
    pipeline = client.pipeline.get_pipeline_by_name(pipeline_name)
    if not pipeline:
        raise Exception(f"Pipeline {pipeline_name} is not existed")

    # 准备流水线参数
    pipeline_parameter_template = environ[pipeline_parameter_env_key]
    run_parameters = json.loads(Template(pipeline_parameter_template).safe_substitute(**kwargs))

    # 确认并触发流水线
    if not Confirm.ask(f"Are you sure to trigger pipeline [red]{pipeline_name}[/red]?"):
        return None

    web_url: str = pipeline._links.additional_properties.get("web").get('href')
    typer.launch(web_url)
    trigger_pipeline: Run = client.pipeline.trigger_pipeline(pipeline_id=pipeline.id, run_parameters=run_parameters)
    return trigger_pipeline.as_dict()



from typing import Union

import typer
from typing_extensions import Annotated

from ..utils.ado import execute_pipeline

app = typer.Typer()


@app.command(
    name="toggle_active_cluster",
    help="切换活动集群",
    short_help="切换活动集群"
)
def toggle_active_cluster(
        release: Annotated[str, typer.Argument(
            help="发布版本号",
            show_default=False,
        )]
) -> Union[dict, None]:
    """
    切换当前的活动集群。

    此命令用于触发 Azure DevOps 流水线来切换活动集群。
    执行后会将流量从当前活动集群切换到非活动集群。

    Args:
        release: 要切换的版本号，例如 "1.0.0"

    环境变量:
        DEPLOY_PIPELINES_TOGGLE_ACTIVE_CLUSTER_NAME: 切换流水线名称
        DEPLOY_PIPELINES_TOGGLE_ACTIVE_CLUSTER_PARAMETERS: 切换流水线参数配置

    Examples:
        切换到指定版本的集群：
            $ hacli deploy toggle_active_cluster 1.0.0

    注意:
        - 此操作会影响生产环境的流量分配，请谨慎执行
        - 确保目标版本已经在非活动集群上部署并测试通过
        - 执行前请确保已正确设置所需的环境变量
    """
    return execute_pipeline(release=release,
                     pipeline_name_env_key="DEPLOY_PIPELINES_TOGGLE_ACTIVE_CLUSTER_NAME",
                     pipeline_parameter_env_key="DEPLOY_PIPELINES_TOGGLE_ACTIVE_CLUSTER_PARAMETERS")



import os

import typer
from typing_extensions import Annotated

app = typer.Typer()


@app.command(name="create_release_note_email_content",
             help="生成 release note 邮件内容",
             short_help="生成邮件内容")
def create_release_note_email_content(
        release: Annotated[str, typer.Argument(help="前一个版本号，例如：1.0.0")],
) -> str:
    return os.environ["RELEASE_NOTE_EMAIL_TEMPLATE"].format(release=release)
